{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uServerBaseForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, StdCtrls, Spin, DDOServer, DDODocument, ComCtrls,
  ExtCtrls, DDOBase, DDODataGrid, DDOProgressBar, DDOUtils, DDOSystem;

const
  WM_ASYNCUPDATE = WM_USER + $6666;

type
  TServerBaseForm = class(TForm)
    DDOListener1: TDDOListener;
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    GridUtil: TDDODataGrid;
    GridThreads: TDDODataGrid;
    GridCalls: TDDODataGrid;
    DDOProgressBar1: TDDOProgressBar;
    ListView1: TListView;
    Label1: TLabel;
    EditPort: TSpinEdit;
    SpeedButtonStart: TSpeedButton;
    SpeedButtonUpdate: TSpeedButton;
    procedure SpeedButtonStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DDOListener1Activate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DDOListener1ClientException(
      Sender: TObject; E: Exception; Method: String);
    procedure DDOListener1RequestProcessed(
      Sender: TObject; Result: TDDODocument);
    procedure GridCallsGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure GridUtilGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure GridThreadsGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure SpeedButtonUpdateClick(Sender: TObject);
  private
    FProcessTimes: TDDOProcessTimes;
    procedure ReadCmdLineParams;
    procedure WMAsyncUpdate(var Message: TMessage); message WM_ASYNCUPDATE;
  protected
    FRequests, FLastValue: Integer;
    procedure RequestUpdate;
    procedure Log(AMessage: String = '');
    procedure UpdateGUI; virtual;
  end;

implementation

{$R *.dfm}

const
  CStarted = 'Started (port %d)';
  CStopped = 'Stopped';
  CException = '%s (%s) %s';

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.RequestUpdate;
begin
  PostMessage(Handle, WM_ASYNCUPDATE, 0, 0);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.WMAsyncUpdate(var Message: TMessage);
var
  lMsg: TMsg;
begin
  while PeekMessage(lMsg, Handle, WM_ASYNCUPDATE, WM_ASYNCUPDATE, PM_REMOVE) do
    ;  // remove duplicate messages
  UpdateGUI;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.SpeedButtonStartClick(Sender: TObject);
begin
  try
    if not DDOListener1.Active then
      DDOListener1.Port := EditPort.Value;
    DDOListener1.Active := not DDOListener1.Active;
  finally
    UpdateGUI;
  end;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.FormCreate(Sender: TObject);
begin
  FProcessTimes := TDDOProcessTimes.Create;
  ReadCmdLineParams;
  ListView1.DoubleBuffered := True;
  UpdateGUI;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.FormDestroy(Sender: TObject);
begin
  DDOListener1.Active := False;
  FreeAndNil(FProcessTimes);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.UpdateGUI;
begin
  SpeedButtonStart.Down := DDOListener1.Active;
  EditPort.Enabled := not DDOListener1.Active;
  EditPort.Value := DDOListener1.Port;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.Log(AMessage: String = '');
begin
  if AMessage = EmptyStr then
    if DDOListener1.Active then
      AMessage := Format(CStarted, [DDOListener1.Port])
    else
      AMessage := CStopped;
  StatusBar1.SimpleText := AMessage
    + ': ' + IntToStr(DDOListener1.Requests);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.DDOListener1Activate(Sender: TObject);
begin
  Log();
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.DDOListener1ClientException(Sender: TObject;
  E: Exception; Method: String);
begin
  Log(Format(CException, [E.ClassName, E.Message, Method]));
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.DDOListener1RequestProcessed(Sender: TObject;
  Result: TDDODocument);
begin
  Inc(FRequests);
  Log();
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.GridCallsGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  Value := (FRequests - FLastValue) * 1000 div GridCalls.Interval;
  FLastValue := FRequests;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.GridUtilGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  DDOProgressBar1.Value := Trunc(FProcessTimes.CPUUsage);
  Value := Trunc(DDOProgressBar1.Value);
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TServerBaseForm.ReadCmdLineParams;
  function nGetValue(Switch: String; Default: Integer): Integer;
  var
    i: Integer;
  begin
    Result := Default;
    for i := 0 to ParamCount -2 do
      if ParamStr(i) = Switch then
        Result := StrToIntDef(ParamStr(i+1), Default);
  end;
begin
  DDOListener1.Port := nGetValue('-p', DDOListener1.Port);
  DDOListener1.Active := FindCmdLineSwitch('a', ['-'], False);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.GridThreadsGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  Value := DDOListener1.ActiveThreads;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TServerBaseForm.ListView1Data(Sender: TObject; Item: TListItem);
begin
  with TDDOServerUtils.GetStubTracker, Item do
  begin
    Caption := Name[Index];
    SubItems.Add(IntToStr(Calls[Index]));
    SubItems.Add(IntToStr(Last[Index]));
    SubItems.Add(IntToStr(Min[Index]));
    SubItems.Add(IntToStr(Max[Index]));
  end;
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:

procedure TServerBaseForm.SpeedButtonUpdateClick(Sender: TObject);
begin
  ListView1.Items.Count := TDDOServerUtils.GetStubTracker.Count;
  ListView1.Invalidate;
end;

end.
